#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

double ask_point_depth(char *str, int i);
double ask_positive_double(char *str);
int ask_positive_int(char *str);
int left_deepest(int start, double *set);
int right_deepest(int start, int finish, double *set);
char ask_for_repeat(void);
void left_turn(int deepest, double interval, double *depth, double *angle);
void right_turn(int deepest, int ending, double interval, double *depth, double *angle);
void show_results(int n, double *depth, double *angle);

int
main()
{
	double interval, max_depth, range;
	double *depth, *angle;
	int n, i, deepest;
	int *res;
	
	max_depth = 0;

	/*primary input block*/
	n = ask_positive_int("Enter the number of points: ");
	interval = ask_positive_double("Enter the interval between points: ");
	range = ask_positive_double("Enter the range to bottom: ");

	/*associating pointers with memory areas*/
	res = malloc(sizeof(*res));
	if (res == NULL) {
		printf("You need more the memory\n");
		exit(5);
	}

	depth = malloc(n*sizeof(*depth));
	if (res == NULL) {
		printf("Memory allocating for depts array failed\n");
		exit(6);
	}
	
	angle = malloc(n*sizeof(*angle));
	if (res == NULL) {
		printf("Memory allocating for angles array failed\n");
		exit(7);
	}

	/*secondary input block*/
	for (i = 0; i < n; i++) {
		depth[i] = ask_point_depth("Enter the depth in point ", i+1);
		
		/*get maximal depth value*/
		if (depth[i] >= max_depth) {
			deepest = i;
			max_depth = depth[i];
		}

		/*increase depth value on range to bottom*/
		depth[i] += range;
	}

	/*these 2 functions are calculating
	angles and depts values in each point*/
	left_turn(deepest, interval, depth, angle);
	right_turn(deepest, n, interval, depth, angle);

	/*output block*/
	show_results(n, depth, angle);

	return 0;
}

/*function getting a non-negative double value
from user printing message *str. Function prints
'i' value if 'i' argument isn't equal 0*/
double
ask_point_depth(char *str, int i)
{
	double value;
	int res;
	char *msg, def;

	def = '\0';
	msg = &def;
	value = -1;

	if (str != NULL)
		if (strlen(str) > 0)
			msg = str;

	while (value < 0) {
		printf(msg);

		if (i != 0) {
			printf("%i: ", i);
		}

		res = scanf("%lf", &value);
		if (res < 1) {
			printf("You entered some trash\n");

			if (ask_for_repeat() == 'y')
				continue;
			exit(1);
		}
		if (value < 0) {
			printf("A depth can't be negative\n");

			if (ask_for_repeat() == 'y')
				continue;
			exit(2);
		}
	}

	return value;
}

/*funcion getting a positive double number
from user printing message *str*/
double
ask_positive_double(char *str)
{
	double value;

	value = -1;

	while (value <= 0) {
		value = ask_point_depth(str, 0);
		if (value == 0) {
			printf("A number must be positive\n");

			if (ask_for_repeat() == 'y')
				continue;
			exit(3);
		}
	}

	return value;
}

/*function getting positive integer number from
user printing message *str*/
int
ask_positive_int(char *str)
{
	double value, difference;

	value = -1;

	while (value < 0) {
		value  = ask_positive_double(str);
		difference = value - (int)value;
		
		if (difference != 0) {
			printf("A number must be integer\n");
			
			if (ask_for_repeat() == 'y')
				continue;
			exit(4);
		}
	}

	return (int)value;
}

/*function returns the index of the deepest point
after the start index*/
int
left_deepest(int start, double *set)
{
	int max = start, i;
	double max_val = 0;

	for (i = start-1; i >= 0; i--) {
		if (set[i] > max_val) {
			max = i;
			max_val = set[i];
		}
	}

	return max;
}

/*function returns the index of the deepest point
before the start index*/
int
right_deepest(int start, int finish, double *set)
{
	int max = start, i;
	double max_val = 0;

	for (i = start+1; i <= finish; i++) {
		if (set[i] > max_val) {
			max = i;
			max_val = set[i];
		}
	}
	
	return max;
}

/*this function asks user to one more attempt*/
char
ask_for_repeat(void)
{
	int res;
	char c;

	while (getchar() != '\n') {}
	printf("Enter 'y' to try again: ");

	res = scanf("%c", &c);
	if (res > 0) {
		res = tolower(c);
		return c;
	}
	return '\0';
}

/*function filling with values a part of depts
and angles arrays before the deepest point*/
void
left_turn(int deepest, double interval, double *depth, double *angle)
{
	/*control height difference for one interval*/
	const double dh0 = tan(M_PI*7/180) * interval;

	/*current height difference between current
	deepest point and smaller deepest point*/
	double dh;

	/*warps variable contains the number of intervals
	at the ends of which we calculate angles and depts*/
	int temp, i, warps;

	do {
		temp = left_deepest(deepest, depth);
		dh = depth[deepest] - depth[temp];
		warps = deepest - temp;

		if (dh < dh0*warps) {
			for (i = (deepest-1); i >= temp; i--) {
				angle[i] = 180*atan2(dh, interval*warps)/M_PI;
				depth[i] = depth[deepest] - dh*(deepest - i)/warps;
			}
		} else {
			for (i = (deepest-1); i >= temp; i--) {
				angle[i] = 7;
				depth[i] = depth[deepest] - dh0*(deepest - i);
			}
		}

		deepest = temp;
	} while (deepest > 0);
}

/*function filling with values a part of depts and angles
arrays after the deepest point including this point*/
void
right_turn(int deepest, int ending, double interval, double *depth, double *angle)
{
	/*control height difference for one interval*/
	const double dh0 = tan(M_PI*7/180) * interval;

	/*current height difference between current
	deepest point and smaller deepest point*/
	double dh;
	
	/*warps variable contains the number of intervals
	at the ends of which we calculate angles and depts*/
	int temp, i, warps;

	do {
		temp = right_deepest(deepest, ending, depth);
		dh = depth[deepest] - depth[temp];
		warps = temp - deepest;

		if (dh < dh0*warps) {
			for (i = deepest; i <= temp; i++) {
				angle[i] = -180*atan2(dh, interval*warps)/M_PI;
				depth[i] = depth[deepest] - dh*(i - deepest)/warps;
			}
		} else {
			for (i = deepest; i <= temp; i++) {
				angle[i] = -7;
				depth[i] = depth[deepest] - dh0*(i - deepest);
			}
		}

		deepest = temp;
	} while (deepest < ending);

}

/*function printing results of calculations*/
void
show_results(int n, double *depth, double *angle)
{
	double start, finish;
	int i;

	/*distances at which the car leaves the first and the last point*/
	start = depth[0]/tan(M_PI*7/180);
	finish = depth[n-1]/tan(M_PI*7/180);

	printf("\nEntrance point is in %.4lf to Point 1. Angle: 7 degrees\n\n", start);

	for (i = 0; i < n; i++) {
		printf("Point %i:\n\tdepth:\t%.4lf\n\t", i+1, depth[i]);

		if (angle[i] > 0) {
			printf("angle:\t%.4lf degrees, machine moves down\n", angle[i]);
		} else if (angle[i] == 0) {
			printf("angle:\t%.4lf degrees, machine moves in parallel the earth\n", angle[i]);
		} else {
			printf("angle:\t%.4lf degrees, machine moves up\n", -angle[i]);
		}
	}

	printf("\nLeaving point is in %.4lf from point %i.\n", finish,n);
	printf("\nEnter 'E' or 'e' for exit\n\n");

	while (tolower(getchar())!= 'e') {}
}
